// all dimesions are in mm

bottom_radius=14;
bottom_height=19;

button_radius=9;
button_hold_length=14.75;
button_hold_width=8.5;
button_hold_height=19;

connector_height= bottom_height / 2;
connector_thickness=2;
connector_thickness_top=2;
connector_filament_diameter=1.75 + .1;
connector_ring_height= connector_height / 2;

degrees_of_notches=30;

hold_length=15;
hold_width=8.75;
hold_height=11;
hold_length_cutout= 16;
hold_width_cutout= 9.75;

notch_length=2.5;
notch_width=2.5;
notch_height=3.5;
notch_height_from_bottom=3;

rail_length=24;
rail_length_inside=12.5;
rail_length_from_center=30;
rail_width=15.25;
rail_height=15.4;
rail_hold_sides=6;
rail_hold_top_bottom=3;

screw_diameter=4;

spring_radius=5.25;
spring_height=20;

top_radius=14;
top_height= rail_height + rail_hold_top_bottom + connector_height + 2;
top_inside_radius= button_radius + .5;
top_button_hold_height= notch_height + notch_height_from_bottom;
top_connector_height= top_height - top_button_hold_height;
top_notch_cutout_width=1.0;
top_notch_cutout_length=.5;

button_height= top_button_hold_height + 10;

$fn=80;




// -------------------------------------------------- Bottom of Joint -----------------------------------------------------------------

translate ([ 0 , 35 , 0 ]) {

	difference () {

// bottom cylinder ---------------------------------------------------------------------------------------------------------------------

		cylinder ( bottom_height , bottom_radius , bottom_radius );

// button hold -------------------------------------------------------------------------------------------------------------------------

		translate ([ - hold_length_cutout / 2 , - hold_width_cutout / 2 , bottom_height - hold_height + .1 ]) {

			cube ([ hold_length_cutout , hold_width_cutout , hold_height ]);}

		translate ([ - hold_width_cutout / 2 , - hold_length_cutout / 2 , bottom_height - hold_height + .1 ]) {

			cube ([ hold_width_cutout , hold_length_cutout , hold_height ]);}

// connector ---------------------------------------------------------------------------------------------------------------------------

		translate ([ 0 , 0 , bottom_height - connector_height ]) {

			difference () {

				cylinder ( connector_height , bottom_radius , bottom_radius );

				cylinder ( connector_height , bottom_radius - connector_thickness - .25 , bottom_radius - connector_thickness - .25 ); 

			}
		}

		translate ([ 0 , 0 , bottom_height - connector_ring_height ]) {

			rotate_extrude ( convexity = 10 )

				translate ([ bottom_radius - connector_filament_diameter , 0 , 10 ]) {

					circle ( connector_filament_diameter / 2 + .15 );

				}
		}

	}

// bottom rail hold ---------------------------------------------------------------------------------------------------------------------

	difference () {

		translate ([ - rail_length_from_center , - ( rail_width + rail_hold_sides ) / 2 , 0 ]) {

			cube ([ rail_length , rail_width + rail_hold_sides , rail_height + rail_hold_top_bottom ]); }

		translate ([ - rail_length_from_center , - rail_width / 2 , rail_hold_top_bottom / 2 ]) {

			cube ([ rail_length_inside , rail_width , rail_height ]);}

		cylinder ( bottom_height , bottom_radius , bottom_radius );

		translate ([ 0 , 0 , bottom_height - connector_height ]) {

			cylinder ( connector_height + .1 , bottom_radius + .5 , bottom_radius + .5 ); }

		translate ([ - rail_length_from_center + 5 , rail_width , ( rail_height + rail_hold_top_bottom ) / 2 ]) {

			rotate ([ 90 , 0 , 0 ]) 

				cylinder ( rail_width * 2 , screw_diameter / 2 , screw_diameter / 2 ); }

	}

// spring holder -----------------------------------------------------------------------------------------------------------------------

	difference () {

		translate ([ 0 , 0 , bottom_height - connector_height - spring_radius - 5 ]) {

			sphere ( spring_radius ); }

		translate ([ - spring_radius / 2 - 5 , - spring_radius / 2 -5 , - spring_radius -10]) {

			cube ( spring_radius + 10 ); }

	}
}

// -------------------------------------------------------------------------------------------------------------------------------------


// --------------------------------------- Button ------------------------------------------------------------------

translate ([ 0 , -33 , 0 ]) {

	cylinder ( button_height , button_radius , button_radius );

// button hold ------------------------------------------------------------------------------------------------------

	difference () {

		translate ([ - hold_length / 2 , - hold_width / 2 , button_height ]) {

			cube ([ hold_length , hold_width , hold_height ]);}

		translate ([ 0 , 0 , button_height ]) {

			cylinder ( hold_height , spring_radius + .25 , spring_radius + .25 );}

	}

	difference () {

		translate ([ - hold_width / 2 , - hold_length / 2 , button_height ]) {

			cube ([ hold_width , hold_length , hold_height ]);}

		translate ([ 0 , 0 , button_height ]) {

			cylinder ( hold_height , spring_radius + .25 , spring_radius + .25 );}

	}

// notches ----------------------------------------------------------------------------------------------------------

	rotate ([ 0 , 0 , 0 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width  , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches * 2 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches * 3 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches * 4 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches * 5 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches * 6 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches * 7 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches * 8 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches * 9 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches * 10 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

	rotate ([ 0 , 0 , degrees_of_notches * 11 ]) {

		translate ([ button_radius - .15 , - notch_width / 2 , top_button_hold_height - notch_height_from_bottom ]) {

			cube ([ notch_length , notch_width , notch_height ]); }

	}

}




// -------------------------------------------------- Top Joint Cover ----------------------------------------------

	difference () {

// outside section cylinder ----------------------------------------------------------------------------------------

	cylinder ( top_height , top_radius , top_radius );

// inside cutouts --------------------------------------------------------------------------------------------------

	cylinder ( top_height , top_inside_radius , top_inside_radius );

	translate ([ 0 , 0 , top_button_hold_height ]) {

		cylinder ( top_connector_height , top_radius - connector_thickness_top , top_radius - connector_thickness + .1 );}

// connector ring --------------------------------------------------------------------------------------------------

		translate ([ 0 , 0 , top_height - connector_ring_height ]) {

			rotate_extrude ( convexity = 10 )

				translate ([ bottom_radius - connector_filament_diameter , 0 , 10 ]) {

					circle ( connector_filament_diameter / 2 + .15 );

				}
		}

	translate ([ top_radius - connector_thickness , 0 , top_height - connector_ring_height ]) {

		rotate ([ 90 , 0 , 0 ]) 

			cylinder ( 10 , connector_filament_diameter / 2 , connector_filament_diameter / 2 ); }

// notches ----------------------------------------------------------------------------------------------------------

	rotate ([ 0 , 0 , 0 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}

	rotate ([ 0 , 0 , degrees_of_notches * 1 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}

	rotate ([ 0 , 0 , degrees_of_notches * 2 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}

	rotate ([ 0 , 0 , degrees_of_notches * 3 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}

	rotate ([ 0 , 0 , degrees_of_notches * 4 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}	

	rotate ([ 0 , 0 , degrees_of_notches * 5 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}

	rotate ([ 0 , 0 , degrees_of_notches * 6 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}

	rotate ([ 0 , 0 , degrees_of_notches * 7 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}

	rotate ([ 0 , 0 , degrees_of_notches * 8 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}

	rotate ([ 0 , 0 , degrees_of_notches * 9 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}

	rotate ([ 0 , 0 , degrees_of_notches * 10 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}

	rotate ([ 0 , 0 , degrees_of_notches * 11 ]) {

		translate ([ button_radius  , - notch_width / 2 - top_notch_cutout_width / 2   , top_button_hold_height - notch_height_from_bottom ]) {

			hull () {

				translate ([ 0 , notch_width / 4 + top_notch_cutout_width / 4 , - notch_height_from_bottom / 2 ]) {

					cube ([ notch_length , notch_width / 2 + top_notch_cutout_width / 2 , .1 ]); }

				translate ([ 0 , 0 , notch_height ]) {

					cube ([ top_notch_cutout_length + notch_length , top_notch_cutout_width + notch_width , .1 ]);}
			}
		}

	}


// end inside cutouts  ----------------------------------------------------------------------------------------------

	}

// top rail hold ----------------------------------------------------------------------------------------------------

	difference () {

		translate ([ - rail_length_from_center , - ( rail_width + rail_hold_sides ) / 2 , 0 ]) {

			cube ([ rail_length , rail_width + rail_hold_sides , rail_height + rail_hold_top_bottom ]); }

		translate ([ - rail_length_from_center , - rail_width / 2 , rail_hold_top_bottom / 2 ]) {

			cube ([ rail_length_inside , rail_width , rail_height ]);}

		cylinder ( bottom_height , bottom_radius , bottom_radius );

		translate ([ 0 , 0 , bottom_height - connector_height ]) {

			cylinder ( connector_height + .1 , bottom_radius , bottom_radius ); }

		translate ([ - rail_length_from_center + 5 , rail_width , ( rail_height + rail_hold_top_bottom ) / 2 ]) {

			rotate ([ 90 , 0 , 0 ]) 

				cylinder ( rail_width * 2 , screw_diameter / 2 , screw_diameter / 2 ); }

	}

// -----------------------------------------------------------------------------------------------------------------

























